-- Class: Plasma effect
-- SDK: Gideros 2012.09.1
-- Author: Andrew Burch
-- Date: 14/10/2012
-- Site: http://www.newhighscore.net
-- Contact: andrew.burch@newhighscore.net
-- Notes: Animated traditional plasma demo effect
--

-- nice pre calculateded cos values 
-- pulled off an old hard drive - cannot remember the source
local preBuiltCosTable = {
	0x4040, 0x4040, 0x4040, 0x4040, 0x3f3f, 0x3f3f, 0x3f3e, 0x3e3e, 0x3e3d, 0x3d3d, 0x3c3c, 0x3b3b,
	0x3b3a, 0x3a39, 0x3938, 0x3837, 0x3736, 0x3535, 0x3434, 0x3332, 0x3231, 0x3030, 0x2f2e, 0x2e2d,
	0x2c2c, 0x2b2a, 0x2929, 0x2827, 0x2625, 0x2524, 0x2322, 0x2221, 0x201f, 0x1e1e, 0x1d1c, 0x1b1b,
	0x1a19, 0x1817, 0x1716, 0x1514, 0x1413, 0x1212, 0x1110, 0x100f, 0x0e0e, 0x0d0c, 0x0c0b, 0x0b0a,
	0x0909, 0x0808, 0x0707, 0x0606, 0x0505, 0x0504, 0x0403, 0x0303, 0x0202, 0x0202, 0x0101, 0x0101,
	0x0100, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0101, 0x0101, 0x0102, 0x0202,
	0x0203, 0x0303, 0x0404, 0x0505, 0x0506, 0x0607, 0x0708, 0x0809, 0x090a, 0x0b0b, 0x0c0c, 0x0d0e,
	0x0e0f, 0x1010, 0x1112, 0x1213, 0x1414, 0x1516, 0x1717, 0x1819, 0x1a1b, 0x1b1c, 0x1d1e, 0x1e1f,
	0x2021 ,0x2222, 0x2324, 0x2525, 0x2627, 0x2829, 0x292a, 0x2b2c, 0x2c2d, 0x2e2e, 0x2f30, 0x3031,
	0x3232, 0x3334, 0x3435, 0x3536, 0x3737, 0x3838, 0x3939, 0x3a3a, 0x3b3b, 0x3b3c, 0x3c3d, 0x3d3d,
	0x3e3e, 0x3e3e, 0x3f3f, 0x3f3f, 0x3f40, 0x4040, 0x4040, 0x4040,
}

-- local constants
local MaxIntensity = 128
local MaxShades = 32
local DoubleMaxShades = MaxShades * 2
local ShadeStep = MaxIntensity / MaxShades

local _tinsert = table.insert
local _newshape = Shape.new

-- keep control values between 1 and 128
local function updateControl(value, mod)
	local newValue = value + mod
	local isUnder = newValue < 1
	local isOverflow = newValue > 128
	local modifier = isUnder and 128 or isOverflow and -128 or 0
	
	newValue = newValue + modifier
	
	return newValue
end



local Plasma = {}
local Plasma_mt = {__index = Plasma}

function Plasma.new(params)
	local cellTable = {}
	local cellWidth = params.cellWidth
	local cellHeight = params.cellHeight
	local xcellCount = params.xcellCount
	local ycellCount = params.ycellCount
	local xoffset = params.xoffset
	local yoffset = params.yoffset
	
	-- build table of rectangles for rendering
	for y = 0, ycellCount-1 do
		local top = yoffset + (y * cellHeight)
		
		for x = 0, xcellCount-1 do
			local left = xoffset + (x * cellWidth)
			local newBlock = _newshape()
			
			newBlock:setLineStyle(0, 0x000000)
			newBlock:setFillStyle(Shape.SOLID, 0xffffff)

			newBlock:beginPath()
			newBlock:moveTo(left, top)
			newBlock:lineTo(left + cellWidth, top)
			newBlock:lineTo(left + cellWidth, top + cellHeight)
			newBlock:lineTo(left, top + cellHeight)
			newBlock:closePath()
			newBlock:endPath()

			stage:addChild(newBlock)

			_tinsert(cellTable, newBlock)
		end
	end
	
	local plasma = {
		cellWidth = cellWidth,
		cellHeight = cellHeight,
		xcellCount = xcellCount,
		ycellCount = ycellCount,
		
		cosTable = preBuiltCosTable,
		cellTable = cellTable,
		
		seed = math.random(0, 128),
	}
	
	return setmetatable(plasma, Plasma_mt)
end

function Plasma:update()
	local seed = self.seed
	local colRef1 = self.colRef1 or 1
	local colRef2 = self.colRef2 or 1
	local colRef3 = self.colRef3 or 1
	local colRef4 = self.colRef4 or 1
	local cosTable = self.cosTable
	local cellTable = self.cellTable
	local ycellCount = self.ycellCount
	local xcellCount = self.xcellCount
	
	local tcol1 = colRef1
	local tcol2 = colRef2
	local tcol3 = 1
	local tcol4 = 1
	
	for y = 0, ycellCount-1 do
		tcol3 = colRef3
		tcol4 = colRef4
		
		local yref = y * xcellCount

		for x = 0, xcellCount-1 do
			local tot = seed + x
			
			tot = tot + cosTable[tcol3]
			tot = tot + cosTable[tcol4]
			tot = tot + cosTable[tcol1]
			tot = tot + cosTable[tcol2]
			tot = tot % 64
			
			local shadeFactor = (tot > MaxShades) and (DoubleMaxShades - tot) or tot
			local val = shadeFactor * ShadeStep
			
			local index = 1 + yref + x
			local rect = cellTable[index]
			rect:setColorTransform(0, 0, val / 256, 1)
			
			tcol3 = updateControl(tcol3, 1)
			tcol4 = updateControl(tcol4, 1)
		end

		tcol1 = updateControl(tcol1, 2)
		tcol2 = updateControl(tcol2, 2)
	end

	self.colRef1 = updateControl(colRef1,-1)
	self.colRef2 = updateControl(colRef2, 1)
	self.colRef3 = updateControl(colRef3,-1)
	self.colRef4 = updateControl(colRef4, 2)
end

return Plasma
